package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.data.CglEnum;
import de.uni_marburg.powersort.data.CompetitionEnum;
import de.uni_marburg.powersort.data.DataEnum;
import de.uni_marburg.powersort.data.IntegerSerializer;
import de.uni_marburg.powersort.sort.SortEnum;

import java.nio.file.Files;
import java.util.List;

import static de.uni_marburg.powersort.data.CompetitionEnum.isCompetitionInput;

public class Filter {
    /* Utility Class */
    private Filter() {
    }

    public static boolean isFiltered(DataEnum d, SortEnum s) {
        // To skip some of the inputs for all sort algorithms, uncomment them here.
//        if (List.of(
//                CglEnum.RANDOM_INTEGERS,
//                CglEnum.ASCENDING_INTEGERS,
//                CglEnum.DESCENDING_INTEGERS,
//                CglEnum.ASCENDING_RUNS,
//                CglEnum.ASCENDING_RUNS_WITH_OVERLAP
//        ).contains(d)) {
//            return true;
//        }

        // To skip all competition inputs, uncomment the next line.
//        if(isCompetitionInput(d)){
//            return true;
//        }

        // Skip some sort algorithms for all competition inputs.
        if(isCompetitionInput(d)){
            if(List.of(
                    SortEnum.BUBBLE_SORT,
                    SortEnum.MERGE_SORT
            ).contains(s)){
                return true;
            }

            if(!Files.exists(IntegerSerializer.DATA_DIR)){
                System.err.println("Folder with inputs of competition missing: " + IntegerSerializer.DATA_DIR.toAbsolutePath());
                return true;
            }
        }

        //
        // Filter based on SortEnum values.
        //

        if (s == SortEnum.QUICK_SORT) {
            return List.of(
                    CglEnum.RANDOM_INTEGERS,
                    CglEnum.ASCENDING_INTEGERS,
                    CglEnum.DESCENDING_INTEGERS,
                    CglEnum.ASCENDING_RUNS,
                    CglEnum.ASCENDING_RUNS_WITH_OVERLAP,

                    CompetitionEnum.COMPETITION_34,
                    CompetitionEnum.COMPETITION_35,
                    CompetitionEnum.COMPETITION_36,
                    CompetitionEnum.COMPETITION_37,
                    CompetitionEnum.COMPETITION_38,
                    CompetitionEnum.COMPETITION_39,
                    CompetitionEnum.COMPETITION_83,
                    CompetitionEnum.COMPETITION_85,
                    CompetitionEnum.COMPETITION_153,
                    CompetitionEnum.COMPETITION_163,
                    CompetitionEnum.COMPETITION_164,
                    CompetitionEnum.COMPETITION_165,
                    CompetitionEnum.COMPETITION_166,
                    CompetitionEnum.COMPETITION_167,
                    CompetitionEnum.COMPETITION_168,
                    CompetitionEnum.COMPETITION_169,
                    CompetitionEnum.COMPETITION_170,
                    CompetitionEnum.COMPETITION_171,
                    CompetitionEnum.COMPETITION_172,
                    CompetitionEnum.COMPETITION_173,
                    CompetitionEnum.COMPETITION_174,
                    CompetitionEnum.COMPETITION_181,
                    CompetitionEnum.COMPETITION_182,
                    CompetitionEnum.COMPETITION_183,
                    CompetitionEnum.COMPETITION_184,
                    CompetitionEnum.COMPETITION_185,
                    CompetitionEnum.COMPETITION_186,
                    CompetitionEnum.COMPETITION_187,
                    CompetitionEnum.COMPETITION_205,
                    CompetitionEnum.COMPETITION_206,
                    CompetitionEnum.COMPETITION_207,
                    CompetitionEnum.COMPETITION_213,
                    CompetitionEnum.COMPETITION_214,
                    CompetitionEnum.COMPETITION_216,
                    CompetitionEnum.COMPETITION_218,
                    CompetitionEnum.COMPETITION_220,
                    CompetitionEnum.COMPETITION_222,
                    CompetitionEnum.COMPETITION_225,
                    CompetitionEnum.COMPETITION_226,
                    CompetitionEnum.COMPETITION_228,
                    CompetitionEnum.COMPETITION_231,
                    CompetitionEnum.COMPETITION_232,
                    CompetitionEnum.COMPETITION_235,
                    CompetitionEnum.COMPETITION_236
            ).contains(d);
        }

        if (s == SortEnum.MERGE_SORT) {
            return List.of(
                    CglEnum.RANDOM_INTEGERS,
                    CglEnum.ASCENDING_INTEGERS,
                    CglEnum.DESCENDING_INTEGERS
            ).contains(d);
        }

        if (s == SortEnum.BUBBLE_SORT) {
            return List.of(
                    CglEnum.DESCENDING_INTEGERS,
                    CglEnum.ASCENDING_RUNS,
                    CglEnum.ASCENDING_RUNS_WITH_OVERLAP
            ).contains(d);
        }

        // TODO: Remove this once performance of FinnSort improved
        if (s == SortEnum.FINN_SORT) {
            return List.of(
                    CglEnum.DESCENDING_INTEGERS,
                    CglEnum.ASCENDING_RUNS,
                    CglEnum.ASCENDING_RUNS_WITH_OVERLAP,

                    CompetitionEnum.COMPETITION_83,
                    CompetitionEnum.COMPETITION_85,
                    CompetitionEnum.COMPETITION_153,
                    CompetitionEnum.COMPETITION_163,
                    CompetitionEnum.COMPETITION_164,
                    CompetitionEnum.COMPETITION_165,
                    CompetitionEnum.COMPETITION_166,
                    CompetitionEnum.COMPETITION_167,
                    CompetitionEnum.COMPETITION_168,
                    CompetitionEnum.COMPETITION_169,
                    CompetitionEnum.COMPETITION_170,
                    CompetitionEnum.COMPETITION_171,
                    CompetitionEnum.COMPETITION_172,
                    CompetitionEnum.COMPETITION_173,
                    CompetitionEnum.COMPETITION_174,
                    CompetitionEnum.COMPETITION_181,
                    CompetitionEnum.COMPETITION_182,
                    CompetitionEnum.COMPETITION_183,
                    CompetitionEnum.COMPETITION_184,
                    CompetitionEnum.COMPETITION_185,
                    CompetitionEnum.COMPETITION_186,
                    CompetitionEnum.COMPETITION_187,
                    CompetitionEnum.COMPETITION_198,
                    CompetitionEnum.COMPETITION_199,
                    CompetitionEnum.COMPETITION_205,
                    CompetitionEnum.COMPETITION_206,
                    CompetitionEnum.COMPETITION_207,
                    CompetitionEnum.COMPETITION_213,
                    CompetitionEnum.COMPETITION_214,
                    CompetitionEnum.COMPETITION_216,
                    CompetitionEnum.COMPETITION_218,
                    CompetitionEnum.COMPETITION_220,
                    CompetitionEnum.COMPETITION_222,
                    CompetitionEnum.COMPETITION_226,
                    CompetitionEnum.COMPETITION_228,
                    CompetitionEnum.COMPETITION_231,
                    CompetitionEnum.COMPETITION_232,
                    CompetitionEnum.COMPETITION_236
            ).contains(d);
        }

        // TODO: Remove this once performance of ASort improved
        if (s == SortEnum.ASORT) {
            return List.of(
                    CglEnum.ASCENDING_RUNS,
                    CglEnum.ASCENDING_RUNS_WITH_OVERLAP,
                    CompetitionEnum.COMPETITION_173,
                    CompetitionEnum.COMPETITION_174
            ).contains(d);
        }

        return false;
    }
}
